@@ -14,7 +14,12 @@
 #'             Passed to \code{boot.ci}.
 #' @param R The number of replications to use for bootstrap.
 #' @param histogram If \code{TRUE}, produces a histogram of bootstrapped values.
-#' @param digits The number of significant digits in the output.              
+#' @param digits The number of significant digits in the output.
+#' @param reportIncomplete If \code{FALSE} (the default),
+#'                         \code{NA} will be reported in cases where there
+#'                         are instances of the calculation of the statistic
+#'                         failing during the bootstrap procedure.
+#' @param verbose If \code{TRUE}, prints additional statistics. 
 #' @param ...    Additional arguments passed to \code{chisq.test}. 
 #' 
 #' @details This modification of Cramer's V
@@ -43,6 +48,9 @@
 #'           the confidence intervals 
 #'           determined by this
 #'           method may not be reliable, or the procedure may fail.
+#'           
+#'           In addition, the function will not return a confidence
+#'           interval if there are zeros in any cell.
 #' 
 #' @author Salvatore Mangiafico, \email{mangiafico@njaes.rutgers.edu}
 #' @references \url{http://rcompanion.org/handbook/H_03.html}
@@ -51,8 +59,6 @@
 #' @return A single statistic, Cramer's V.  
 #'         Or a small data frame consisting of Cramer's V,
 #'         and the lower and upper confidence limits.
-#'         In the data frame, \code{r} 
-#'         is just a duplicate of the Cramer's V value.
 #'         
 #' @examples
 #' ### Equal probabilities example
@@ -83,41 +89,72 @@
 
 cramerVFit = function(x, p=rep(1/length(x), length(x)), 
                       ci=FALSE, conf=0.95, type="perc",
-                      R=1000, histogram=FALSE, digits=4, ...) {
+                      R=1000, histogram=FALSE, digits=4,
+                     reportIncomplete=FALSE, 
+                      verbose=FALSE, ...) {
   CV=NULL
   N = sum(x)
   Chi.sq = suppressWarnings(chisq.test(x=x, p=p, ...)$statistic)
   K   = length(x)
   CV =  sqrt(Chi.sq/N/(K-1))
   
+    if(verbose){
+     cat("\n")
+     cat("Number of cells =", signif(K, digits=digits))
+     cat("\n")
+     cat("N               =", signif(N, digits=digits))
+     cat("\n")
+     cat("Chi-squared     =", signif(Chi.sq, digits=digits))
+     cat("\n")
+     cat("V               =", signif(CV, digits=digits))
+     cat("\n")
+     cat("\n")
+    }
+  
   CV = signif(as.numeric(CV), digits=digits)
   
+  if(is.nan(CV) & ci==TRUE){
+    return(data.frame(Cramer.V=CV, lower.ci=NA, upper.ci=NA))}
+  
+  if(any(x==0) & ci==TRUE){
+    return(data.frame(Cramer.V=CV, lower.ci=NA, upper.ci=NA))} 
+  
 if(ci==TRUE){
     Counts = as.data.frame(x)
     Long=data.frame(x = rep(row.names(Counts), Counts$x))
     rownames(Long) = seq(1:nrow(Long))
     
-  Function = function(input, index){
-             Input = input[index,]
-             
-  Obs = as.vector(table(Input))
-  CV=NULL
-  N = sum(Obs)
-  Chi.sq = suppressWarnings(chisq.test(x=Obs, p=p, ...)$statistic)
-  K   = length(Obs)
-  CV =  sqrt(Chi.sq/N/(K-1))
-  
-  CV = signif(as.numeric(CV), digits=digits)
-  
-  return(CV)
+    L1     = length(x)
+    
+    Function = function(input, index){
+      Input = input[index,]
+      
+      NOTEQUAL=0
+      
+      if(length(unique(Input)) != L1){NOTEQUAL=1}
+      
+      if(NOTEQUAL==1){FLAG=1; return(c(NA,FLAG))}
+      
+      if(NOTEQUAL==0){
+         Obs    = as.vector(table(Input))
+         CV     = NULL
+         N      = sum(Obs)
+         Chi.sq = suppressWarnings(chisq.test(x=Obs, p=p, ...)$statistic)
+         K      = length(Obs)
+         CV     = sqrt(Chi.sq/N/(K-1))
+         CV     = signif(as.numeric(CV), digits=digits)
+         FLAG   = 0  
+         return(c(CV,FLAG))}
   }
 
   Boot = boot(Long, Function, R=R)
   BCI  = boot.ci(Boot, conf=conf, type=type)
-  if(type=="norm") {CI1=BCI$normal[2];  CI2=BCI$normal[3];}
-  if(type=="basic"){CI1=BCI$basic[4];   CI2=BCI$basic[5];}
-  if(type=="perc") {CI1=BCI$percent[4]; CI2=BCI$percent[5];}
-  if(type=="bca")  {CI1=BCI$bca[4];     CI2=BCI$bca[5];}  
+  if(type=="norm") {CI1=BCI$normal[2];  CI2=BCI$normal[3]}
+  if(type=="basic"){CI1=BCI$basic[4];   CI2=BCI$basic[5]}
+  if(type=="perc") {CI1=BCI$percent[4]; CI2=BCI$percent[5]}
+  if(type=="bca")  {CI1=BCI$bca[4];     CI2=BCI$bca[5]}
+
+  if(sum(Boot$t[,2])>0 & reportIncomplete==FALSE) {CI1=NA; CI2=NA}
   
   CI1=signif(CI1, digits=digits)
   CI2=signif(CI2, digits=digits)
@@ -127,5 +164,5 @@ if(ci==TRUE){
   
 }
  if(ci==FALSE){names(CV)="Cramer V"; return(CV)}
- if(ci==TRUE){return(data.frame(Cramer.V=CV, lower.ci=CI1, upper.ci=CI2, r=CV))}  
+ if(ci==TRUE){return(data.frame(Cramer.V=CV, lower.ci=CI1, upper.ci=CI2))}  
 }
